@@ -28,6 +28,10 @@ void BVS_Sampler::loglikelihood(
     arma::vec alphas_Rowsum;
     if(proportion_model)
     {
+        #ifdef _OPENMP
+        #pragma omp parallel for
+        #endif
+
         for(unsigned int l=0; l<L; ++l)
         {
             alphas.col(l) = arma::exp( zetas(0, l) + dataclass.datX.slice(l) * zetas.submat(1, l, p, l) );
@@ -90,7 +94,9 @@ void BVS_Sampler::loglikelihood(
     //return loglik;
 }
 
+// NO NEED 'loglikelihood0()', SINCE ALL ARE BASED ON JOINT LIKELIHOOD 'loglikelihood()'
 // log-density for survival data only
+/*
 void BVS_Sampler::loglikelihood0(
     const arma::vec& xi,
     const arma::mat& zetas,
@@ -167,6 +173,7 @@ void BVS_Sampler::loglikelihood0(
 
     //return loglik;
 }
+*/
 
 void BVS_Sampler::sampleGamma(
     arma::umat& gammas_,
@@ -176,7 +183,7 @@ void BVS_Sampler::sampleGamma(
     unsigned int& gamma_acc_count_,
     arma::vec& log_likelihood_,
 
-    const armsParmClass armsPar,
+    const armsParmClass& armsPar,
     void *hyperpar_,
 
     const arma::vec& xi_,
@@ -184,16 +191,16 @@ void BVS_Sampler::sampleGamma(
     arma::mat& betas_,
     double kappa_,
     double tau0Sq_,
-    arma::vec tauSq_,
+    const arma::vec& tauSq_,
     double pi0,
 
     bool proportion_model,
 
-    double& logPosteriorBeta,
+    // double& logPosteriorBeta,
     arma::mat& datProportion,
     arma::vec& datTheta,
-    arma::mat datMu,
-    arma::mat weibullS,
+    const arma::mat& datMu,
+    const arma::mat& weibullS,
     const DataClass &dataclass)
 {
 
@@ -253,18 +260,18 @@ void BVS_Sampler::sampleGamma(
         double pi = pi0;
         for(auto i: updateIdx)
         {
-            if(pi0 == 0.)
+            if(pi0 == 0.) // this is to control if using pre-defined constant pi0 or hyperprior
             {
                 //// feature-specific Bernoulli probability
-                pi = R::rbeta(hyperpar->piA + (double)(proposedGamma(i,componentUpdateIdx)),
-                                hyperpar->piB + (double)(p) - (double)(proposedGamma(i,componentUpdateIdx)));
+                // pi = R::rbeta(hyperpar->piA + (double)(proposedGamma(i,componentUpdateIdx)),
+                //                 hyperpar->piB + 1.0 - (double)(proposedGamma(i,componentUpdateIdx)));
                 //// column-specific Bernoulli probability
                 // pi = R::rbeta(hyperpar->piA + (double)(arma::sum(proposedGamma.col(componentUpdateIdx))),
                 //                 hyperpar->piB + (double)(p) - (double)(arma::sum(proposedGamma.col(componentUpdateIdx))));
+                //// row-specific Bernoulli probability
+                pi = R::rbeta(hyperpar->piA + (double)(arma::accu(gammas_.row(i))),
+                                    hyperpar->piB + (double)(L) - (double)(arma::accu(gammas_.row(i))));
             }
-            //// row-specific Bernoulli probability
-            // double pi = R::rbeta(hyperpar->piA + (double)(arma::accu(proposedGamma.row(i))),
-            //                     hyperpar->piB + (double)(L) - (double)(arma::accu(proposedGamma.row(i))));
             // logProposalGammaRatio += logPDFBernoulli( proposedGamma(i,componentUpdateIdx), pi_proposed ) - logPDFBernoulli( gammas_(i,componentUpdateIdx), pi[componentUpdateIdx] );
             proposedGammaPrior(i,componentUpdateIdx) = logPDFBernoulli( proposedGamma(i,componentUpdateIdx), pi );
             logProposalGammaRatio +=  proposedGammaPrior(i, componentUpdateIdx) - logP_gamma_(i, componentUpdateIdx);
@@ -307,6 +314,10 @@ void BVS_Sampler::sampleGamma(
                 }
             }
             */
+            #ifdef _OPENMP
+            #pragma omp parallel for default(shared) reduction(+:logProposalGammaRatio)
+            #endif
+
             for(unsigned int i=0; i<hyperpar->mrfG_edge_n; ++i)
             {
                 if( mrfG(i, 0) != mrfG(i, 1))
@@ -343,7 +354,7 @@ void BVS_Sampler::sampleGamma(
 
     // update betas based on the proposal gammas
     arma::mat betas_proposal = betas_;
-    double logPosteriorBeta_proposal = 0.;
+    // double logPosteriorBeta_proposal = 0.;
     // double logPosteriorBeta = 0.; //TODO: logPosteriorBeta will be passed through function argument
     // double logProposalBetaRatio = 0.;
 
@@ -378,8 +389,6 @@ void BVS_Sampler::sampleGamma(
         betas_proposal,
         tau0Sq_,
         tauSq_[componentUpdateIdx],
-        hyperpar->tauA,
-        hyperpar->tauB,
 
         proposedGamma,
 
@@ -388,8 +397,8 @@ void BVS_Sampler::sampleGamma(
         datMu,
         datProportion,
         weibullS,
-        dataclass,
-        logPosteriorBeta_proposal
+        dataclass
+        // logPosteriorBeta_proposal
     );
 
     // double logPriorBetaRatio = 0.; // perhaps no need in our Bayesian GPTCM??? >> We need!!!
@@ -551,7 +560,7 @@ void BVS_Sampler::sampleGamma(
         }
         log_likelihood_ = proposedLikelihood;
         betas_ = betas_proposal;
-        logPosteriorBeta = logPosteriorBeta_proposal;
+        // logPosteriorBeta = logPosteriorBeta_proposal;
 
         ++gamma_acc_count_;
     }
@@ -604,7 +613,7 @@ void BVS_Sampler::sampleEta(
     unsigned int& eta_acc_count_,
     arma::vec& log_likelihood_,
 
-    const armsParmClass armsPar,
+    const armsParmClass& armsPar,
     void *hyperpar_,
     arma::mat& zetas_,
     const arma::mat& betas_,
@@ -616,10 +625,10 @@ void BVS_Sampler::sampleEta(
 
     bool dirichlet,
 
-    double& logPosteriorZeta,
+    // double& logPosteriorZeta,
     arma::vec& datTheta,
-    arma::mat weibullS,
-    arma::mat weibullLambda,
+    const arma::mat& weibullS,
+    arma::mat& weibullLambda,
     const DataClass &dataclass)
 {
 
@@ -681,11 +690,14 @@ void BVS_Sampler::sampleEta(
             if(rho0 == 0.)
             {
                 //// feature-specific Bernoulli probablity
-                rho = R::rbeta(hyperpar->rhoA + (double)(proposedEta(i,componentUpdateIdx)),
-                                hyperpar->rhoB + (double)(p) - (double)(proposedEta(i,componentUpdateIdx)));
+                // rho = R::rbeta(hyperpar->rhoA + (double)(proposedEta(i,componentUpdateIdx)),
+                //                 hyperpar->rhoB + 1.0 - (double)(proposedEta(i,componentUpdateIdx)));
                 //// column-specific Bernoulli probability
                 // rho = R::rbeta(hyperpar->rhoA + (double)(arma::sum(proposedEta.col(componentUpdateIdx))),
                 //                 hyperpar->rhoB + (double)(p) - (double)(arma::sum(proposedEta.col(componentUpdateIdx))));
+                //// row-specific Bernoulli probablity
+                rho = R::rbeta(hyperpar->rhoA + (double)(arma::accu(etas_.row(i))),
+                                    hyperpar->rhoB + (double)(L) - (double)(arma::accu(etas_.row(i))));
             }
             //// the following random-walk MH sampler result in increasing rho 
             // double proposedRho = std::exp( std::log( rho(i, componentUpdateIdx) ) + R::rnorm(0.0, 1.0) );
@@ -707,9 +719,6 @@ void BVS_Sampler::sampleEta(
             //         logP_eta_(i, componentUpdateIdx) = proposedEtaPrior0;
             //     }
             // }
-            //// row-specific Bernoulli probablity
-            // double rho = R::rbeta(hyperpar->rhoA + (double)(arma::accu(proposedEta.row(i))),
-            //                     hyperpar->rhoB + (double)(L) - (double)(arma::accu(proposedEta.row(i))));
             // logProposalEtaRatio += logPDFBernoulli( proposedEta(i,componentUpdateIdx), rho_proposed ) - logPDFBernoulli( etas_(i,componentUpdateIdx), rho[componentUpdateIdx] );
             proposedEtaPrior(i,componentUpdateIdx) = logPDFBernoulli( proposedEta(i,componentUpdateIdx), rho );
             logProposalEtaRatio +=  proposedEtaPrior(i, componentUpdateIdx) - logP_eta_(i, componentUpdateIdx);
@@ -738,6 +747,10 @@ void BVS_Sampler::sampleEta(
 
         if((updateIdxMRF_common.n_elem > 0) && (hyperpar->mrfB_prop > 0))
         {
+            #ifdef _OPENMP
+            #pragma omp parallel for default(shared) reduction(+:logProposalEtaRatio)
+            #endif
+
             for(unsigned int i=0; i<hyperpar->mrfG_prop_edge_n; ++i)
             {
                 if( mrfG(i, 0) != mrfG(i, 1))
@@ -761,7 +774,7 @@ void BVS_Sampler::sampleEta(
 
     // update other quantities related to acceptance ratio
     arma::mat zetas_proposal = zetas_;
-    double logPosteriorZeta_proposal = 0.;
+    // double logPosteriorZeta_proposal = 0.;
     // double logProposalZetaRatio = 0.;
 
     // Do we need to update \xi, \zeta, \kappa based on proposal betas & gammas?
@@ -799,10 +812,6 @@ void BVS_Sampler::sampleEta(
         zetas_proposal, // This is different from 'zetas_' in previous arms_gibbs_zeta()
         w0Sq_,
         wSq_[componentUpdateIdx],
-        // hyperpar->w0A,
-        // hyperpar->w0B,
-        hyperpar->wA,
-        hyperpar->wB,
         proposedEta, // This is different from 'etas_' in previous arms_gibbs_zeta()
 
         kappa_,
@@ -810,8 +819,8 @@ void BVS_Sampler::sampleEta(
         datTheta,
         weibullS,
         weibullLambda,
-        dataclass,
-        logPosteriorZeta_proposal
+        dataclass
+        // logPosteriorZeta_proposal
     );
 
     /// NO NEED TO CALCULATE logPriorZetaRatio & logProposalZetaRatio, SINCE THEY WILL BE CANCELLED OUT IN OUR MODEL.
@@ -862,7 +871,7 @@ void BVS_Sampler::sampleEta(
         }
         log_likelihood_ = proposedLikelihood;
         zetas_ = zetas_proposal;
-        logPosteriorZeta = logPosteriorZeta_proposal;
+        // logPosteriorZeta = logPosteriorZeta_proposal;
 
         ++eta_acc_count_;
     }
@@ -1034,8 +1043,6 @@ double BVS_Sampler::gammaBanditProposal(
                                 logPDFBernoulli(mutantGamma(updateIdx(i),componentUpdateIdx_),banditZeta(updateIdx(i)));
         }
 
-        // note that above I might be resampling a value equal to the current one, thus not updating da facto ... TODO
-
         // Compute logProposalRatio probabilities
         // normalised_mismatch_backwards = normalised_mismatch_backwards - Utils::logspace_add(normalised_mismatch_backwards);
         // normalised_mismatch_backwards = normalised_mismatch_backwards / arma::as_scalar(arma::sum(normalised_mismatch_backwards));
@@ -1094,7 +1101,7 @@ double BVS_Sampler::etaBanditProposal(
     // normalised_mismatch2 = mismatch2 / arma::as_scalar(arma::sum(mismatch2));
     normalised_mismatch2 = mismatch2 /arma::sum(mismatch2);
 
-    // TODO: confirm if this is "ε-Greedy Strategy for Bernoulli Bandits". Choose ε=0.5 to balance exploration and exploitation
+    // Use "ε-Greedy Strategy for Bernoulli Bandits". Choose ε=0.5 to balance exploration and exploitation
     if( R::runif(0,1) < 0.5 )   // one deterministic update
     {
         // Decide which to update
@@ -1144,8 +1151,6 @@ double BVS_Sampler::etaBanditProposal(
                                 logPDFBernoulli(mutantEta(updateIdx(i),componentUpdateIdx_),banditZeta2(updateIdx(i)));
         }
 
-        // note that above I might be resampling a value equal to the current one, thus not updating da facto ... TODO
-
         // Compute logProposalRatio probabilities
         // normalised_mismatch_backwards = normalised_mismatch_backwards - Utils::logspace_add(normalised_mismatch_backwards);
         // normalised_mismatch_backwards2 = normalised_mismatch_backwards2 / arma::as_scalar(arma::sum(normalised_mismatch_backwards2));
@@ -1211,7 +1216,6 @@ unsigned int BVS_Sampler::randWeightedIndexSampleWithoutReplacement(
     return t;
 }
 
-// TODO: verify if the following function is equivalent to 'arma::sum( arma::log(weights.elem( indexes )) )'? It seems NO! Why?
 // logPDF rand Weighted Indexes (need to implement the one for the original starting vector?)
 double BVS_Sampler::logPDFWeightedIndexSampleWithoutReplacement(
     const arma::vec& weights,
