@@ -13,12 +13,10 @@
 //' @param npoint Maximum number of envelope points
 //'
 void ARMS_Gibbs::arms_gibbs_xi(
-    const armsParmClass armsPar,
+    const armsParmClass& armsPar,
     arma::vec& currentPars,
     double v0Sq,
     double vSq,
-    // double vA,
-    // double vB,
     arma::mat datProportion,
     arma::mat weibullS,
     const DataClass &dataclass)
@@ -41,12 +39,12 @@ void ARMS_Gibbs::arms_gibbs_xi(
 
     // // double xinit[armsPar.ninit];
     std::vector<double> xinit(armsPar.ninit); // Use std::vector instead of VLA to avoid warning about varying length of array
-    if (!armsPar.simple)
-    {
-        arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
-        for (unsigned int i = 0; i < armsPar.ninit; ++i)
-            xinit[i] = xinit0[i];
-    }
+    // if (!armsPar.simple)
+    // {
+    // }
+    arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
+    for (unsigned int i = 0; i < armsPar.ninit; ++i)
+        xinit[i] = xinit0[i];
 
     //dataS *mydata = create_mydata(currentPars, j, vA, vB, datX0, datProportion, datEvent, weibullS);
     dataS *mydata = (dataS *)malloc(sizeof (dataS));
@@ -73,6 +71,11 @@ void ARMS_Gibbs::arms_gibbs_xi(
     // for (int i = 0; i < armsPar.n; ++i)
     // {
     // Gibbs sampling
+
+    #ifdef _OPENMP
+    #pragma omp parallel for 
+    #endif
+
     for (unsigned int j = 0; j < p; ++j)
     {
         mydata->jj = j;
@@ -169,14 +172,10 @@ void ARMS_Gibbs::arms_gibbs_xi(
 //' @param npoint Maximum number of envelope points
 //'
 void ARMS_Gibbs::arms_gibbs_beta(
-    const armsParmClass armsPar,
+    const armsParmClass& armsPar,
     arma::mat& currentPars,
     arma::vec& tauSq,
     double& tau0Sq,
-    double tauA,
-    double tauB,
-    double tau0A,
-    double tau0B,
 
     // const arma::umat& gammas,
     arma::umat gammas,
@@ -189,8 +188,9 @@ void ARMS_Gibbs::arms_gibbs_beta(
     // const arma::cube datX,
     // const arma::uvec datEvent,
     // const arma::vec datTime,
-    const DataClass &dataclass,
-    double& logPosteriorBeta)
+    const DataClass &dataclass
+    // double& logPosteriorBeta
+)
 {
     /* make a subfunction arms_gibbs for only vector betas that can be used for (varying-length) variable selected vector*/
 
@@ -199,7 +199,7 @@ void ARMS_Gibbs::arms_gibbs_beta(
     unsigned int p = dataclass.datX.n_cols;
     unsigned int L = dataclass.datX.n_slices;
 
-    logPosteriorBeta = 0.; // reset value 0
+    // logPosteriorBeta = 0.; // reset value 0
 
     // objects for arms()
     double minD = armsPar.betaMin;
@@ -215,12 +215,12 @@ void ARMS_Gibbs::arms_gibbs_beta(
 
     // double xinit[armsPar.ninit];
     std::vector<double> xinit(armsPar.ninit); // Use std::vector instead of VLA
-    if (!armsPar.simple)
-    {
-        arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
-        for (unsigned int i = 0; i < armsPar.ninit; ++i)
-            xinit[i] = xinit0[i];
-    }
+    // if (!armsPar.simple)
+    // {
+    // }
+    arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
+    for (unsigned int i = 0; i < armsPar.ninit; ++i)
+        xinit[i] = xinit0[i];  
 
     dataS *mydata = (dataS *)malloc(sizeof (dataS));
 
@@ -234,7 +234,7 @@ void ARMS_Gibbs::arms_gibbs_beta(
     // mydata->xr = xr;
     // mydata->vA = vA;
     // mydata->vB = vB;
-    // mydata->tauSq = tauSq;
+    mydata->tau0Sq = tau0Sq;
     mydata->kappa = kappa;
     mydata->datTheta = datTheta.memptr();
     mydata->datMu = datMu.memptr();
@@ -245,15 +245,13 @@ void ARMS_Gibbs::arms_gibbs_beta(
 
     // arma::vec tauSq_tmp = arma::ones<arma::vec>(L);
 
+    // not easy to parallize the following for-loop due to data dependencies
     for (unsigned int l = 0; l < L; ++l)
     {
         // Gibbs sampling
         // mydata->tauSq = sampleW(tauA, tauB, currentPars.col(l));
         // mydata->tau0Sq = sampleW0(tauA, tauB, currentPars(0,l));
-        
-        tau0Sq = sampleW(tau0A, tau0B, currentPars.row(0).t());
-        mydata->tau0Sq = tau0Sq;
-        tauSq[l] = sampleW(tauA, tauB, currentPars.submat(1,l,p,l));
+
         mydata->tauSq = tauSq[l];
         // tauSq_tmp[l] = sampleW(tauA, tauB, currentPars.col(l));
         // mydata->tauSq = tauSq_tmp[l];
@@ -297,23 +295,23 @@ void ARMS_Gibbs::arms_gibbs_beta(
                 int neval, ncent = 0;
 
                 int err;
-                if (armsPar.simple)
-                {
-                    err = ARMS::arms_simple (
-                              armsPar.ninit, &minD, &maxD,
-                              EvalFunction::log_dens_betas, mydata,
-                              armsPar.metropolis, &xprev, xsamp.data());
-                }
-                else
-                {
-                    double convex = armsPar.convex;
+                // if (armsPar.simple)
+                // {
+                //     err = ARMS::arms_simple (
+                //               armsPar.ninit, &minD, &maxD,
+                //               EvalFunction::log_dens_betas, mydata,
+                //               armsPar.metropolis, &xprev, xsamp.data());
+                // }
+                // else
+                // {
+                // }
+                double convex = armsPar.convex;
                     err = ARMS::arms (
                               xinit.data(), armsPar.ninit, &minD, &maxD,
                               EvalFunction::log_dens_betas, mydata,
                               &convex, armsPar.npoint,
                               armsPar.metropolis, &xprev, xsamp.data(),
                               armsPar.nsamp, qcent, xcent, ncent, &neval);
-                }
 
                 // check ARMS validity
                 if (err > 0)
@@ -355,12 +353,10 @@ void ARMS_Gibbs::arms_gibbs_beta(
 // Multivariate ARMS via Gibbs sampler for betaK; used for M-H sampling for gammas update
 void ARMS_Gibbs::arms_gibbs_betaK(
     const unsigned int k,
-    const armsParmClass armsPar,
+    const armsParmClass& armsPar,
     arma::mat& currentPars,
     double tau0Sq,
     double tauSqK,
-    double tauA,
-    double tauB,
 
     // const arma::umat& gammas,
     arma::umat gammas,
@@ -370,8 +366,9 @@ void ARMS_Gibbs::arms_gibbs_betaK(
     arma::mat datMu,
     arma::mat& datProportion,
     arma::mat weibullS,
-    const DataClass &dataclass,
-    double& logPosteriorBeta)
+    const DataClass &dataclass
+    // double& logPosteriorBeta
+)
 {
     /* make a subfunction arms_gibbs for only vector betas that can be used for (varying-length) variable selected vector*/
 
@@ -380,19 +377,19 @@ void ARMS_Gibbs::arms_gibbs_betaK(
     unsigned int p = dataclass.datX.n_cols;
     unsigned int L = dataclass.datX.n_slices;
 
-    logPosteriorBeta = 0.; // reset value 0
+    // logPosteriorBeta = 0.; // reset value 0
 
     // objects for arms()
     double minD = armsPar.betaMin;
     double maxD = armsPar.betaMax;
 
     std::vector<double> xinit(armsPar.ninit); // Use std::vector instead of VLA
-    if (!armsPar.simple)
-    {
-        arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
-        for (unsigned int i = 0; i < armsPar.ninit; ++i)
-            xinit[i] = xinit0[i];
-    }
+    // if (!armsPar.simple)
+    // {
+    // }
+    arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
+    for (unsigned int i = 0; i < armsPar.ninit; ++i)
+        xinit[i] = xinit0[i];
 
     dataS *mydata = (dataS *)malloc(sizeof (dataS));
 
@@ -439,23 +436,23 @@ void ARMS_Gibbs::arms_gibbs_betaK(
             int neval, ncent = 0;
 
             int err;
-            if (armsPar.simple)
-            {
-                err = ARMS::arms_simple (
-                          armsPar.ninit, &minD, &maxD,
-                          EvalFunction::log_dens_betas, mydata,
-                          armsPar.metropolis, &xprev, xsamp.data());
-            }
-            else
-            {
-                double convex = armsPar.convex;
-                err = ARMS::arms (
+            // if (armsPar.simple)
+            // {
+            //     err = ARMS::arms_simple (
+            //               armsPar.ninit, &minD, &maxD,
+            //               EvalFunction::log_dens_betas, mydata,
+            //               armsPar.metropolis, &xprev, xsamp.data());
+            // }
+            // else
+            // {
+            // }
+            double convex = armsPar.convex;
+            err = ARMS::arms (
                           xinit.data(), armsPar.ninit, &minD, &maxD,
                           EvalFunction::log_dens_betas, mydata,
                           &convex, armsPar.npoint,
                           armsPar.metropolis, &xprev, xsamp.data(),
                           armsPar.nsamp, qcent, xcent, ncent, &neval);
-            }
 
             // check ARMS validity
             if (err > 0)
@@ -485,15 +482,10 @@ void ARMS_Gibbs::arms_gibbs_betaK(
 //' @param dirichlet Not yet implemented
 //'
 void ARMS_Gibbs::arms_gibbs_zeta(
-    const armsParmClass armsPar,
+    const armsParmClass& armsPar,
     arma::mat& currentPars,
     double& w0Sq,
     arma::vec& wSq,
-    // bool w0IGamma,
-    double w0A,
-    double w0B,
-    double wA,
-    double wB,
 
     arma::umat etas,
 
@@ -501,9 +493,10 @@ void ARMS_Gibbs::arms_gibbs_zeta(
     bool dirichlet,
     arma::vec& datTheta,
     arma::mat weibullS,
-    arma::mat weibullLambda,
-    const DataClass &dataclass,
-    double& logPosteriorZeta)
+    arma::mat& weibullLambda,
+    const DataClass &dataclass
+    // double& logPosteriorZeta
+)
 {
     // arma::mat datProportion, // remove this argument
     // double phi,
@@ -514,7 +507,7 @@ void ARMS_Gibbs::arms_gibbs_zeta(
     unsigned int p = dataclass.datX.n_cols;
     unsigned int L = dataclass.datX.n_slices;
 
-    logPosteriorZeta = 0.;
+    // logPosteriorZeta = 0.;
     // int armsPar.metropolis = metropolis;
 
     // objects for arms()
@@ -529,12 +522,12 @@ void ARMS_Gibbs::arms_gibbs_zeta(
 
     // double xinit[armsPar.ninit];
     std::vector<double> xinit(armsPar.ninit); // Use std::vector instead of VLA
-    if (!armsPar.simple)
-    {
-        arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
-        for (unsigned int i = 0; i < armsPar.ninit; ++i)
-            xinit[i] = xinit0[i];
-    }
+    // if (!armsPar.simple)
+    // {
+    // }
+    arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
+    for (unsigned int i = 0; i < armsPar.ninit; ++i)
+        xinit[i] = xinit0[i];
 
     if (!dirichlet)
         Rprintf("Warning: In arms_gibbs_zeta(), Dirichlet modeling with logit/alr-link is not implement!\n");
@@ -547,7 +540,7 @@ void ARMS_Gibbs::arms_gibbs_zeta(
     mydata->p = p;
     mydata->L = L;
     mydata->N = N;
-    // mydata->w0Sq = w0Sq;
+    mydata->w0Sq = w0Sq;
     // mydata->wSq = wSq;
     //mydata->phi = phi;
     //mydata->dirichlet = dirichlet,
@@ -567,9 +560,6 @@ void ARMS_Gibbs::arms_gibbs_zeta(
     {
         // Gibbs sampling
         // mydata->w0Sq = sampleW0(w0A, w0B, currentPars(0,l));
-        w0Sq = sampleW(w0A, w0B, currentPars.row(0).t());
-        mydata->w0Sq = w0Sq;
-        wSq[l] = sampleW(wA, wB, currentPars.submat(1,l,p,l));
         mydata->wSq = wSq[l];
         // wSq_tmp[l] = sampleW(wA, wB, currentPars.submat(1,l,p,l));
         // mydata->wSq = wSq_tmp[l];
@@ -596,23 +586,23 @@ void ARMS_Gibbs::arms_gibbs_zeta(
                 int neval, ncent = 0;
 
                 int err;
-                if (armsPar.simple)
-                {
-                    err = ARMS::arms_simple (
-                              armsPar.ninit, &minD, &maxD,
-                              EvalFunction::log_dens_zetas, mydata,
-                              armsPar.metropolis, &xprev, xsamp.data());
-                }
-                else
-                {
-                    double convex = armsPar.convex;
-                    err = ARMS::arms (
+                // if (armsPar.simple)
+                // {
+                //     err = ARMS::arms_simple (
+                //               armsPar.ninit, &minD, &maxD,
+                //               EvalFunction::log_dens_zetas, mydata,
+                //               armsPar.metropolis, &xprev, xsamp.data());
+                // }
+                // else
+                // {
+                // }
+                double convex = armsPar.convex;
+                err = ARMS::arms (
                               xinit.data(), armsPar.ninit, &minD, &maxD,
                               EvalFunction::log_dens_zetas, mydata,
                               &convex, armsPar.npoint,
                               armsPar.metropolis, &xprev, xsamp.data(),
                               armsPar.nsamp, qcent, xcent, ncent, &neval);
-                }
 
                 // check ARMS validity
                 if (err > 0)
@@ -652,15 +642,10 @@ void ARMS_Gibbs::arms_gibbs_zeta(
 // Multivariate ARMS via Gibbs sampler for betaK; used for M-H sampling for gammas update
 void ARMS_Gibbs::arms_gibbs_zetaK(
     const unsigned int k,
-    const armsParmClass armsPar,
+    const armsParmClass& armsPar,
     arma::mat& currentPars,
     double w0Sq,
     double wSqK,
-    // bool w0IGamma,
-    // double w0A,
-    // double w0B,
-    double wA,
-    double wB,
 
     arma::umat etas,
 
@@ -668,9 +653,10 @@ void ARMS_Gibbs::arms_gibbs_zetaK(
     bool dirichlet,
     arma::vec& datTheta,
     arma::mat weibullS,
-    arma::mat weibullLambda,
-    const DataClass &dataclass,
-    double& logPosteriorZeta)
+    arma::mat& weibullLambda,
+    const DataClass &dataclass
+    // double& logPosteriorZeta
+)
 {
 
     // dimensions
@@ -678,19 +664,19 @@ void ARMS_Gibbs::arms_gibbs_zetaK(
     unsigned int p = dataclass.datX.n_cols;
     unsigned int L = dataclass.datX.n_slices;
 
-    logPosteriorZeta = 0.;
+    // logPosteriorZeta = 0.;
 
     // objects for arms()
     double minD = armsPar.zetaMin;
     double maxD = armsPar.zetaMax;
 
     std::vector<double> xinit(armsPar.ninit); // Use std::vector instead of VLA
-    if (!armsPar.simple)
-    {
-        arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
-        for (unsigned int i = 0; i < armsPar.ninit; ++i)
-            xinit[i] = xinit0[i];
-    }
+    // if (!armsPar.simple)
+    // {
+    // }
+    arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
+    for (unsigned int i = 0; i < armsPar.ninit; ++i)
+        xinit[i] = xinit0[i];
 
     if (!dirichlet)
         Rprintf("Warning: In arms_gibbs_zeta(), Dirichlet modeling with logit/alr-link is not implement!\n");
@@ -730,23 +716,23 @@ void ARMS_Gibbs::arms_gibbs_zetaK(
             int neval, ncent = 0;
 
             int err;
-            if (armsPar.simple)
-            {
-                err = ARMS::arms_simple (
-                          armsPar.ninit, &minD, &maxD,
-                          EvalFunction::log_dens_zetas, mydata,
-                          armsPar.metropolis, &xprev, xsamp.data());
-            }
-            else
-            {
-                double convex = armsPar.convex;
-                err = ARMS::arms (
+            // if (armsPar.simple)
+            // {
+            //     err = ARMS::arms_simple (
+            //               armsPar.ninit, &minD, &maxD,
+            //               EvalFunction::log_dens_zetas, mydata,
+            //               armsPar.metropolis, &xprev, xsamp.data());
+            // }
+            // else
+            // {
+            // }
+            double convex = armsPar.convex;
+            err = ARMS::arms (
                           xinit.data(), armsPar.ninit, &minD, &maxD,
                           EvalFunction::log_dens_zetas, mydata,
                           &convex, armsPar.npoint,
                           armsPar.metropolis, &xprev, xsamp.data(),
                           armsPar.nsamp, qcent, xcent, ncent, &neval);
-            }
 
             // check ARMS validity
             if (err > 0)
@@ -775,7 +761,7 @@ void ARMS_Gibbs::arms_gibbs_zetaK(
 //' @param dirichlet Not yet implemented
 //'
 void ARMS_Gibbs::arms_kappa(
-    const armsParmClass armsPar,
+    const armsParmClass& armsPar,
     double& currentPars,
     double kappaA,
     double kappaB,
@@ -801,12 +787,12 @@ void ARMS_Gibbs::arms_kappa(
 
     // double xinit[armsPar.ninit];
     std::vector<double> xinit(armsPar.ninit); // Use std::vector instead of VLA
-    if (!armsPar.simple)
-    {
-        arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
-        for (unsigned int i = 0; i < armsPar.ninit; ++i)
-            xinit[i] = xinit0[i];
-    }
+    // if (!armsPar.simple)
+    // {
+    // }
+    arma::vec xinit0 = arma::linspace( minD+1.0e-10, maxD-1.0e-10, armsPar.ninit );
+    for (unsigned int i = 0; i < armsPar.ninit; ++i)
+        xinit[i] = xinit0[i];
 
     dataS *mydata = (dataS *)malloc(sizeof (dataS));
     //mydata->currentPars = &currentPars;
