@@ -6,6 +6,14 @@
 #include "BVS.h"
 #include "global.h"
 
+#ifdef _OPENMP
+ extern omp_lock_t RNGlock; /*defined in global.h*/
+ #include <omp.h>
+#endif
+
+#include <Rcpp.h>
+// [[Rcpp::plugins(openmp)]]
+
 #include <RcppArmadillo.h>
 // [[Rcpp::depends(RcppArmadillo)]]
 
@@ -20,13 +28,12 @@
 //' @param n number of samples to draw
 //' @param nsamp how many samples to draw for generating each sample; only the last draw will be kept
 //' @param ninit number of initials as meshgrid values for envelop search
-//' @param metropolis value 0/1 for metropolis step or not
-//' @param simple logical value for implementing a simple arms algorithm
 //' @param convex adjustment for convexity (non-negative value, default 1.0)
 //' @param npoint maximum number of envelope points
 //' @param dirichlet not yet implemented
 //' @param proportion_model logical value for modeling the proportions data
 //' @param BVS logical value for implementing Bayesian variable selection
+//' @param threads maximum threads used for parallelization. Default is 1
 //' @param gamma_prior one of \code{c("bernoulli", "MRF")}
 //' @param gamma_sampler one of \code{c("mc3", "bandit")}
 //' @param eta_prior one of \code{c("bernoulli", "MRF")}
@@ -49,28 +56,41 @@ Rcpp::List run_mcmc(
     unsigned int n,
     int nsamp,
     int ninit,
-    int metropolis,
-    bool simple,
     double convex,
     int npoint,
     bool dirichlet,
     bool proportion_model,
     bool BVS,
+    int threads,
     const std::string& gamma_prior,
     const std::string& gamma_sampler,
     const std::string& eta_prior,
     const std::string& eta_sampler,
 
-    Rcpp::List initList,
-    Rcpp::List rangeList,
-    Rcpp::List hyperparList,
+    const Rcpp::List& initList,
+    const Rcpp::List& rangeList,
+    const Rcpp::List& hyperparList,
 
-    arma::uvec datEvent,
-    arma::vec datTime,
-    arma::cube datX,
-    arma::mat datX0,
-    arma::mat datProportionConst)
+    const arma::uvec& datEvent,
+    const arma::vec& datTime,
+    const arma::cube& datX,
+    const arma::mat& datX0,
+    const arma::mat& datProportionConst)
 {
+    #ifdef _OPENMP
+        // omp_set_nested( 0 );
+        // omp_set_num_threads( 1 );
+    if( threads == 1 ){
+        omp_set_nested( 0 );
+        omp_set_num_threads( 1 );
+    } else {
+            omp_init_lock(&RNGlock);  // init RNG lock for the parallel part
+        
+            omp_set_nested(0); // 1=enable, 0=disable nested parallelism (e.g. compute likelihoods in parallel at least wrt to outcomes + wrt to individuals)
+            omp_set_num_threads( threads ); // TODO: 'threads' seems not faster always 
+    }
+    #endif
+
     // dimensions
     unsigned int N = datX.n_rows;
     unsigned int p = datX.n_cols;
@@ -85,14 +105,15 @@ Rcpp::List run_mcmc(
     // input constant data sets in a class
     DataClass dataclass(datEvent, datTime, datX, datX0, datProportionConst);
     // ,datProportion, datTheta, datMu, weibullLambda, weibullS);
-    datEvent.clear();
-    datTime.clear();
-    datX.clear();
-    datX0.clear();
-    datProportionConst.clear();
+    // datEvent.clear();
+    // datTime.clear();
+    // datX.clear();
+    // datX0.clear();
+    // datProportionConst.clear();
 
     // arms parameters in a class
-    armsParmClass armsPar(n, nsamp, ninit, metropolis, simple, convex, npoint,
+    int metropolis = 1; // this is fixed, since arms.simple without metropolis step seems useless.
+    armsParmClass armsPar(n, nsamp, ninit, metropolis, convex, npoint,
                           Rcpp::as<double>(rangeList["xiMin"]),
                           Rcpp::as<double>(rangeList["xiMax"]),
                           Rcpp::as<double>(rangeList["zetaMin"]),
@@ -101,10 +122,10 @@ Rcpp::List run_mcmc(
                           Rcpp::as<double>(rangeList["kappaMax"]),
                           Rcpp::as<double>(rangeList["betaMin"]),
                           Rcpp::as<double>(rangeList["betaMax"]));
-    rangeList = Rcpp::List();  // Clear it by creating a new empty List
+    // rangeList = Rcpp::List();  // Clear it by creating a new empty List
 
     // hyperparameters
-    // NOTE: Do not change try to change the struct hyperparS to a C++ class until I become a super expert in C++,
+    // NOTE: Do not try to change the struct hyperparS to a C++ class now,
     // because that will change a lot the related code toward the original arms C code in change C++ class does not fit well.
     hyperparS *hyperpar = (hyperparS *)malloc(sizeof (hyperparS));
 
@@ -179,7 +200,7 @@ Rcpp::List run_mcmc(
     hyperpar->kappaB = Rcpp::as<double>(hyperparList["kappaB"]);
     hyperpar->kappaIGamma = Rcpp::as<bool>(hyperparList["kappaIGamma"]);
 
-    hyperparList = Rcpp::List();  // Clear it by creating a new empty List
+    // hyperparList = Rcpp::List();  // Clear it by creating a new empty List
 
     // Gamma Sampler
     Gamma_Sampler_Type gammaSampler;
@@ -229,7 +250,7 @@ Rcpp::List run_mcmc(
     arma::mat zetas = Rcpp::as<arma::mat>(initList["zetas"]);
     arma::mat betas = Rcpp::as<arma::mat>(initList["betas"]);
     double kappa = Rcpp::as<double>(initList["kappa"]);
-    initList = Rcpp::List();  // Clear it by creating a new empty List
+    // initList = Rcpp::List();  // Clear it by creating a new empty List
 
     unsigned int nIter_thin = nIter / thin;
     // initializing mcmc results
@@ -415,6 +436,8 @@ Rcpp::List run_mcmc(
         dataclass,
         log_likelihood
     );
+    loglikelihood_mcmc.row(0) = log_likelihood.t();
+    /*
     arma::vec log_likelihood0;
     BVS_Sampler::loglikelihood0(
         xi,
@@ -426,11 +449,11 @@ Rcpp::List run_mcmc(
         dataclass,
         log_likelihood0
     );
-    // loglikelihood_mcmc.row(0) = log_likelihood.t();
     loglikelihood_mcmc.row(0) = log_likelihood0.t();
+    */
 
-    double logPosteriorBeta = 0.; // TODO: If using this, it's better to be computed through likelihood()
-    double logPosteriorZeta = 0.;
+    // double logPosteriorBeta = 0.; // TODO: If using this, it's better to be computed through likelihood()
+    // double logPosteriorZeta = 0.;
 
     // // initial points for ARS approach
     // arma::vec initialPoints = arma::linspace( range->xiMin+0.1, range->xiMin-0.1, ninit );
@@ -458,6 +481,14 @@ Rcpp::List run_mcmc(
                         std::string(cTotalLength * (1. - (m + 1.) / nIter), '-') << // printing empty part
                         "] " << (int)((m + 1.) / nIter * 100.0) << "%\r";             // printing percentage
 
+
+        // update \beta's variacnes tau0Sq, tauSq
+        tau0Sq = sampleV(hyperpar->tau0A, hyperpar->tau0B, betas.row(0).t());
+        for (unsigned int l=0; l<L; ++l)
+        {
+            tauSq[l] = sampleV(hyperpar->tauA, hyperpar->tauB, betas.submat(1,l,p,l));
+        }
+
         // update \xi's variance vSq
         v0Sq = sampleV0(hyperpar->v0A, hyperpar->v0B, xi[0]);
         // hyperpar->v0Sq = v0Sq;
@@ -513,6 +544,12 @@ Rcpp::List run_mcmc(
                 // Here both etas and zetas are updated inside due to passing their addresses
                 if(BVS)
                 {
+                    // update \zeta's variacnes w0Sq, wSq
+                    w0Sq = sampleV(hyperpar->w0A, hyperpar->w0B, zetas.row(0).t());
+                    for (unsigned int l=0; l<L; ++l)
+                    {
+                        wSq[l] = sampleV(hyperpar->wA, hyperpar->wB, zetas.submat(1,l,p,l));
+                    }
                     //// update cluster-specific Bernoulli probability via Gibbs
                     // arma::vec rho = arma::zeros<arma::vec>(L); 
                     // for (unsigned int l=0; l<L; ++l)
@@ -541,7 +578,7 @@ Rcpp::List run_mcmc(
                         rho0,
                         dirichlet,
 
-                        logPosteriorZeta,
+                        // logPosteriorZeta,
                         datTheta,
                         weibullS,
                         weibullLambda,
@@ -562,15 +599,8 @@ Rcpp::List run_mcmc(
                 ARMS_Gibbs::arms_gibbs_zeta(
                     armsPar,
                     zetas,
-                    // hyperpar->w0Sq,
-                    // hyperpar->wSq,
                     w0Sq,
                     wSq,
-                    // hyperpar->w0IGamma,
-                    hyperpar->w0A,
-                    hyperpar->w0B,
-                    hyperpar->wA,
-                    hyperpar->wB,
                     etas,
 
                     kappa,
@@ -578,8 +608,8 @@ Rcpp::List run_mcmc(
                     datTheta,
                     weibullS,
                     weibullLambda,
-                    dataclass,
-                    logPosteriorZeta
+                    dataclass
+                    // logPosteriorZeta
                 );
 
                 // zeta_mcmc.row(1+m) = arma::vectorise(zetas).t();
@@ -656,7 +686,7 @@ Rcpp::List run_mcmc(
 
                 proportion_model,
 
-                logPosteriorBeta,
+                // logPosteriorBeta,
                 datProportion,
                 datTheta,
                 datMu,
@@ -675,10 +705,6 @@ Rcpp::List run_mcmc(
             // hyperpar->tauSq,
             tauSq,
             tau0Sq,
-            hyperpar->tauA,
-            hyperpar->tauB,
-            hyperpar->tau0A,
-            hyperpar->tau0B,
 
             gammas,
 
@@ -687,8 +713,8 @@ Rcpp::List run_mcmc(
             datMu,
             datProportion,
             weibullS,
-            dataclass,
-            logPosteriorBeta
+            dataclass
+            // logPosteriorBeta
         );
 
         // }
@@ -698,6 +724,10 @@ Rcpp::List run_mcmc(
         // hyperpar->tauSq = sampleTau(hyperpar->tauA, hyperpar->tauB, betas);
         // tauSq_mcmc[1+m] = hyperpar->tauSq;
 
+        #ifdef _OPENMP
+        #pragma omp parallel for
+        #endif
+        
         // update Weibull's quantities based on the new betas
         for(unsigned int l=0; l<L; ++l)
         {
@@ -736,6 +766,7 @@ Rcpp::List run_mcmc(
             beta_mcmc.row(1+nIter_thin_count) = arma::vectorise(betas).t();
             tauSq_mcmc[1+nIter_thin_count] = tauSq[0];//hyperpar->tauSq; // TODO: only keep the firs one for now
 
+            /*
             // update likelihood0
             BVS_Sampler::loglikelihood0(
                 xi,
@@ -746,7 +777,7 @@ Rcpp::List run_mcmc(
                 dataclass,
                 log_likelihood0
             );
-            loglikelihood_mcmc.row(1+nIter_thin_count) = log_likelihood0.t();
+            */
             if(BVS)
             {
                 gamma_mcmc.row(1+nIter_thin_count) = arma::vectorise(gammas).t();
@@ -754,23 +785,25 @@ Rcpp::List run_mcmc(
                 {
                     eta_mcmc.row(1+nIter_thin_count) = arma::vectorise(etas).t();
                 }
+            } 
+            else 
+            {
+                // update likelihood
+                BVS_Sampler::loglikelihood(
+                    xi,
+                    zetas,
+                    betas,
+                    kappa,
+                    proportion_model,
+                    dataclass,
+                    log_likelihood
+                );
             }
+            // save loglikelihoods
+            loglikelihood_mcmc.row(1+nIter_thin_count) = log_likelihood.t();
+
             ++nIter_thin_count;
         }
-        if(!BVS)
-        {
-            // update likelihood
-            BVS_Sampler::loglikelihood(
-                xi,
-                zetas,
-                betas,
-                kappa,
-                proportion_model,
-                dataclass,
-                log_likelihood
-            );
-        }
-
 
     }
 
